class MjoolnException(Exception):
    """
    Parent for all module specific exceptions
    """
    pass


class PixieInPipeline(MjoolnException):
    """
    Raised by code mistakes if environment variable
    ``MJOOLN__PIXIE_IN_PIPELINE='true'``
    """
    pass


class AngryElf(MjoolnException):
    """
    Raised when ``elf()`` is unable to figure out what you are trying to do.
    It usually does not take well to not being able to do magic
    """
    pass


class CryptError(MjoolnException):
    """
    Rased by :class:`.Crypt`, mainly when password or crypt_key is invalid
    """
    pass


class BadSeed(MjoolnException):
    """
    Raised by :class:`.Seed`
    """
    pass


class DicError(MjoolnException):
    """
    Raised by :class:`.Dic`
    """
    pass


class DocError(MjoolnException):
    """
    Raised by :class:`.Doc`
    """
    pass


class DocumentError(DocError):
    """
    Raised by :class:`.Document`
    """
    pass


class IdentityError(MjoolnException):
    """
    Raised by :class:`.Identity`
    """
    pass


class BadWord(MjoolnException):
    """
    Raised by :class:`.Word`
    """
    pass


class NotAnInteger(BadWord):
    """
    Raised by :class:`.Word` when trying to get an integer from a non-integer
    word
    """
    pass


class InvalidKey(MjoolnException):
    """
    Raised by :class:`.Key`
    """
    pass


class ZuluError(MjoolnException):
    """
    Raised by :class:`.Zulu`
    """
    pass


class AtomError(MjoolnException):
    """
    Raised by :class:`.Atom`
    """
    pass


class PathError(MjoolnException):
    """
    Raised by :class:`.Path`
    """
    pass


class FolderError(MjoolnException):
    """
    Raised by :class:`.Folder`
    """
    pass


class FileError(MjoolnException):
    """
    Raised by :class:`.File`
    """
    pass


class ArchiveError(MjoolnException):
    """
    Raised by :class:`.Archive`
    """
    pass


class StoreError(MjoolnException):
    """
    Raised by :class:`.Store`
    """
    pass


import os

#: Home folder. Default is '~/.mjooln'
HOME = os.getenv('MJOOLN__HOME', '~/.mjooln').replace('\\', '/')

#: Default values. Override by adding MJOOLN__ first, and put in environment
#: variables
DEFAULT = {
    'PIXIE': 'false',
    'MINIMUM_WORD_LENGTH': 1,
    'WORD_SEPARATOR': '__',
    'CLASS_SEPARATOR': '___',
    'COMPRESSED_EXTENSION': 'gz',
    'ENCRYPTED_EXTENSION': 'aes',
    'ZULU_TO_ISO': 'true',
    'TRUNK__PATH': os.path.join(HOME, 'trunk.yaml').replace('\\', '/'),
    'TRUNK__EXTENSION': 'yaml',
    'TRUNK__AUTO_SCAN': 'true',
    'TRUNK__AUTO_SCAN_FOLDERS': '',
}


def get_env(name):
    public_name = 'MJOOLN__' + name
    return os.getenv(public_name, DEFAULT[name])


#: When flag is set to 'true', the Pixie will slow down your code, but
#: in return be very picky about your mistakes
PIXIE = get_env('PIXIE') == 'true'

#: Minimum word length. Default is 1
MINIMUM_WORD_LENGTH = get_env('MINIMUM_WORD_LENGTH')

#: Word separator. Default is double underscore
WORD_SEPARATOR = get_env('WORD_SEPARATOR')

#: Class separator. Default is triple underscore
CLASS_SEPARATOR = get_env('CLASS_SEPARATOR')

#: Compressed (reserved) extension. Default is '.gz'
COMPRESSED_EXTENSION = get_env('COMPRESSED_EXTENSION')

#: Encrypted (reserved) extension. Default is '.aes'
ENCRYPTED_EXTENSION = get_env('ENCRYPTED_EXTENSION')

#: Flags converting Zulu to iso string when creating doc
ZULU_TO_ISO = get_env('ZULU_TO_ISO') == 'true'



__version__ = '0.6.0.13'

from mjooln.exception import *
from mjooln.core import Crypt, Glass, Math, Text, Seed, \
    JSON, YAML, Dic, Doc, Tic, Word, Identity, Key, Zulu, Atom, Waiter, \
    Path, Folder, File, Archive

# Experimental classes
# from mjooln.experimental.system import System
# from mjooln.experimental.document import Document
# from mjooln.experimental.store import Store, DocStore, ConfigStore
